/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.location.rule.impl;

import com.yiring.app.domain.location.LocationAlarmRule;
import com.yiring.app.domain.location.LocationAlarmRuleRepository;
import com.yiring.app.param.location.rule.LocationAlarmRuleAddParam;
import com.yiring.app.param.location.rule.LocationAlarmRuleModifyParam;
import com.yiring.app.param.location.rule.LocationAlarmRuleParam;
import com.yiring.app.service.location.rule.LocationAlarmRuleService;
import com.yiring.app.util.JpaUtil;
import com.yiring.app.vo.location.rule.LocationAlarmRuleVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

/**
 * @author tml
 * @version 1.0
 * @date 2022/5/5 16:59
 */
@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class LocationAlarmRuleServiceImpl implements LocationAlarmRuleService {

    @Resource
    private LocationAlarmRuleRepository locationAlarmRuleRepository;

    @Override
    public Result<String> addRule(LocationAlarmRuleAddParam param) {
        LocationAlarmRule locationAlarmRule = param.transform();
        locationAlarmRuleRepository.saveAndFlush(locationAlarmRule);
        return Result.ok();
    }

    @Override
    public Result<String> modifyRule(LocationAlarmRuleModifyParam param) {
        LocationAlarmRule locationAlarmRule = param.transform();
        Optional<LocationAlarmRule> optional = locationAlarmRuleRepository.findById(param.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "要修改的规则不存在");
        }
        JpaUtil.copyNotNullProperties(locationAlarmRule, optional.get());
        return Result.ok();
    }

    @Override
    public Result<String> removeRule(IdParam idParam) {
        Optional<LocationAlarmRule> optional = locationAlarmRuleRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "要修改的规则不存在");
        }
        optional.get().setDeleted(true);
        return Result.ok();
    }

    @Override
    public Result<LocationAlarmRuleVo> findById(IdParam idParam) {
        Optional<LocationAlarmRule> optional = locationAlarmRuleRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "查询的规则不存在");
        }
        LocationAlarmRuleVo ruleVo = LocationAlarmRuleVo.transform(optional.get());
        return Result.ok(ruleVo);
    }

    @Override
    public Result<PageVo<LocationAlarmRuleVo>> findByCondition(LocationAlarmRuleParam param, PageParam pageParam) {
        Pageable pageable = PageParam.toPageable(pageParam);
        Specification<LocationAlarmRule> specification = (root, query, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            if (param.getMapId() != null) {
                list.add(criteriaBuilder.equal(root.get("mapId"), param.getMapId()));
            }
            if (param.getFenceId() != null) {
                list.add(criteriaBuilder.equal(root.get("locationFence"), param.getFenceId()));
            }
            Predicate[] array = list.toArray(new Predicate[0]);
            return criteriaBuilder.and(array);
        };
        Set<Integer> informManner = param.getInformManner();
        if (CollectionUtils.isEmpty(informManner)) {
            Page<LocationAlarmRule> rulePage = locationAlarmRuleRepository.findAll(specification, pageable);
            List<LocationAlarmRuleVo> voList = rulePage
                .get()
                .map(LocationAlarmRuleVo::transform)
                .collect(Collectors.toList());
            PageVo<LocationAlarmRuleVo> pageVo = PageVo.build(voList, rulePage.getTotalElements());
            return Result.ok(pageVo);
        }
        List<LocationAlarmRule> list = locationAlarmRuleRepository.findAll(specification, pageable.getSort());
        List<LocationAlarmRule> ruleList = list
            .stream()
            .filter(e -> {
                for (Integer item : informManner) {
                    boolean contains = e.getInformManner().contains(item.toString());
                    if (!contains) {
                        return false;
                    }
                }
                return true;
            })
            .collect(Collectors.toList());
        int pageBegin = (pageParam.getPageNo() - 1) * pageParam.getPageSize();
        int pageEnd = pageBegin + 10;
        if (pageBegin > ruleList.size()) {
            return Result.no(Status.EXPECTATION_FAILED, "分页参数异常");
        }
        if (pageEnd > ruleList.size()) {
            pageEnd = ruleList.size();
        }
        List<LocationAlarmRule> pageList = ruleList.subList(pageBegin, pageEnd);
        List<LocationAlarmRuleVo> page = pageList
            .stream()
            .map(LocationAlarmRuleVo::transform)
            .collect(Collectors.toList());
        PageVo<LocationAlarmRuleVo> pageVo = PageVo.build(page, ruleList.size());
        return Result.ok(pageVo);
    }
}
