/* (C) 2022 YiRing, Inc. */
package com.yiring.app.web.location.rule;

import com.yiring.app.domain.alarm.AlarmType;
import com.yiring.app.param.location.rule.LocationAlarmRuleAddParam;
import com.yiring.app.param.location.rule.LocationAlarmRuleModifyParam;
import com.yiring.app.param.location.rule.LocationAlarmRuleParam;
import com.yiring.app.service.location.rule.LocationAlarmRuleService;
import com.yiring.app.vo.location.rule.LocationAlarmRuleVo;
import com.yiring.common.core.Result;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.Objects;
import javax.annotation.Resource;
import javax.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.Sort;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author tml
 * @version 1.0
 * @date 2022/5/5 14:19
 */
@Slf4j
@Validated
@Api(tags = "AlarmRule(电子围栏报警规则,配置报警给谁)")
@RestController
@RequestMapping("/alarm/rule")
public class LocationAlarmRuleController {

    @Resource
    private LocationAlarmRuleService locationAlarmRuleService;

    @ApiOperation(value = "添加一条电子围栏报警规则")
    @PostMapping("/addRule")
    public Result<String> addRule(@Valid LocationAlarmRuleAddParam param) {
        return locationAlarmRuleService.addRule(param);
    }

    @ApiOperation(value = "修改一条电子围栏报警规则")
    @PostMapping("/modifyRule")
    public Result<String> modifyRule(@Valid LocationAlarmRuleModifyParam param) {
        return locationAlarmRuleService.modifyRule(param);
    }

    @ApiOperation(value = "删除一条电子围栏报警规则")
    @PostMapping("/removeRule")
    public Result<String> removeRule(@Valid IdParam idParam) {
        return locationAlarmRuleService.removeRule(idParam);
    }

    @ApiOperation(value = "根据id查询一条电子围栏报警规则")
    @PostMapping("/findById")
    public Result<LocationAlarmRuleVo> findById(@Valid IdParam idParam) {
        return locationAlarmRuleService.findById(idParam);
    }

    @ApiOperation(value = "根据条件查询电子围栏报警规则")
    @PostMapping("/findByCondition")
    public Result<PageVo<LocationAlarmRuleVo>> findByCondition(
        @Valid LocationAlarmRuleParam param,
        @Valid PageParam pageParam
    ) {
        if (Objects.isNull(pageParam.getSortField())) {
            pageParam.setSortField(AlarmType.Fields.createTime);
            pageParam.setSortOrder(Sort.Direction.DESC);
        }
        return locationAlarmRuleService.findByCondition(param, pageParam);
    }
}
